package com.alipay.scicreditdata;

package ..deTemp.1733360930114;

import com.alipay.scicreditdata.FsV1CalculateUtil;
import com.alipay.scicreditdata.FsV2CalculateUtil;
import com.alipay.scicreditdata.FsV2Constants;
import com.alipay.scicreditdata.parser.zx.chain.HandlerChainV2;
import com.alipay.scicreditdata.parser.zx.domain.result.PersonCreditReport;
import com.alipay.scicreditdata.parser.zx.exception.PbocException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import shade.x.org.apache.commons.lang3.StringEscapeUtils;
import shade.x.org.apache.commons.lang3.tuple.Pair;

public class CreditV2XmlUtil {
  private static final List<String> DEFAULT_DOUBLE_VAR = Arrays.asList(new String[] { 
        FsV2Constants.WJQ_FZFDK_ACCT_3M_RATE, FsV2Constants.LOW_BIZ_TYPE_ORG_RATE, FsV2Constants.LOW_TERMS_TYPE_ORG_RATE_12M, FsV2Constants.YH_QUERY_ORG_CNT_RATE, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_RATE_3M_12M, FsV2Constants.QRY_TOT_CNT_RATE_1M_3M, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_RATE_3M_12M, FsV2Constants.QRY_TOT_CNT_RATE_3M_6M, FsV2Constants.HIGH_QUERY_ORG_ACCT_CNT_RATE_12M, FsV2Constants.HIGN_QUERY_ORG_ACCT_CNT_RATE_12M_24, 
        FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_RATE_6M_12M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_RATE_3M_6M, FsV2Constants.HIGN_QUERY_ORG_ACCT_CNT_RATE_6M_12, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_RATE_6M_12M, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_RATE_3M_6M, FsV2Constants.QRY_ORG_TP_XFJRGS_ORGS_RATE_3M_12M, FsV2Constants.R1_AVG_WJQ_ZH_TERMS });
  
  private static final List<String> DEFAULT_LONG_VAR = Arrays.asList(new String[] { 
        FsV2Constants.R1_TOT_CREDIT_RATE_GE_20_ACCT_CNT, FsV2Constants.R1_TOT_CREDIT_RATE_GE_80_ACCT_CNT, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_3M, FsV2Constants.R4_TOT_ACCT_CNT, FsV2Constants.R4_TOT_ORGS, FsV2Constants.QRY_TOT_RSN_DKSP_ORGS_1M, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_6M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_3M, FsV2Constants.QRY_TOT_CNT_1M, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_12M, 
        FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_6M, FsV2Constants.QRY_TOT_CNT_RATE_1M_12M, FsV2Constants.WJQ_GRXFDK_LST_TERMS_GE_6_ACCT_CNT, FsV2Constants.QRY_TOT_CNT_3M, FsV2Constants.HIGH_TERMS_TYPE_ACCT_CNT, FsV2Constants.WJQ_FZFDK_LST_TERMS_CNT, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_12M, FsV2Constants.TOT_QRY_MTH_6M, FsV2Constants.TOT_QRY_MTH_12M, FsV2Constants.HIGN_QUERY_ORG_CNT, 
        FsV2Constants.QRY_TOT_CNT_6M, FsV2Constants.QRY_TOT_ORG_TP_YHJG_ORGS_12M, FsV2Constants.R1_TOT_XK_ACCT_CNT_12M, FsV2Constants.HIGN_QUERY_ORG_ACCT_CNT_12, FsV2Constants.QRY_TOT_CNT_12M, FsV2Constants.R1_TOT_WJQ_YYE_ACCT_CNT, FsV2Constants.TOT_ACCT_CNT, FsV2Constants.R1_TOT_CREDIT_RATE_GE_50_ORGS, FsV2Constants.R1_TOT_CREDIT_RATE_GE_20_ORGS, FsV2Constants.R1_TOT_ORGS, 
        FsV2Constants.D1_WJQ_FZFDK_GE_10K_ACCT_CNT, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_CNT_12M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_CNT_3M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_CNT_6M, FsV2Constants.QRY_TOT_RSN_DBZGSC_CNT_24M, FsV2Constants.R1_WJQ_FZFDK_LE_3K_ORGS, FsV2Constants.R2_TOT_ACTV_ZHZ_CREDIT_RATE_GE_80_ACCT_CNT, FsV2Constants.R1_WJQ_FZFDK_GE_10K_ORGS, FsV2Constants.QRY_TOT_ORG_YHJG_XYKSP_ORGS_12M });
  
  public static final List<String> FS_88 = Arrays.asList(new String[] { 
        FsV2Constants.YH_QUERY_ORG_CNT_RATE, FsV2Constants.WJQ_TERMS_AVG, FsV2Constants.WJQ_GRXFDK_TERMS_AVG, FsV2Constants.WJQ_GRXFDK_LST_TERMS_GE_6_ACCT_CNT, FsV2Constants.WJQ_FZFDK_LST_TERMS_CNT, FsV2Constants.WJQ_FZFDK_ACCT_3M_RATE, FsV2Constants.TOT_QRY_MTH_6M, FsV2Constants.TOT_QRY_MTH_12M, FsV2Constants.TOT_ACCT_CNT, FsV2Constants.R4_TOT_YYHK_MTH_CNT_6M, 
        FsV2Constants.R4_TOT_ORGS, FsV2Constants.R4_TOT_ACCT_CNT, FsV2Constants.R2_TOT_ACTV_ZHZ_CREDIT_RATE_GE_80_ACCT_CNT, FsV2Constants.R2_TOT_ACCT_ST_HY_REP_AMT, FsV2Constants.R2_TOT_ACCT_ST_HY_ACCT_USD_AMT, FsV2Constants.R2_MAX_ACCT_ST_HY_ACCT_USD_RATE, FsV2Constants.R2_AVG_CREDIT_RATE_GE_50_ACCT_CNT, FsV2Constants.R2_AVG_ACCT_ST_HY_REP_AMT, FsV2Constants.R1_WJQ_FZFDK_LE_3K_ORGS, FsV2Constants.R1_WJQ_FZFDK_GE_10K_ORGS, 
        FsV2Constants.R1_TOT_XK_ACCT_CNT_12M, FsV2Constants.R1_TOT_WJQ_YYE_ACCT_CNT, FsV2Constants.R1_TOT_WJQ_WJFLGZJYS_ACCT_CNT, FsV2Constants.R1_TOT_ORGS, FsV2Constants.R1_TOT_CREDIT_RATE_GE_80_ACCT_CNT, FsV2Constants.R1_TOT_CREDIT_RATE_GE_50_ORGS, FsV2Constants.R1_TOT_CREDIT_RATE_GE_50_ACCT_CNT, FsV2Constants.R1_TOT_CREDIT_RATE_GE_20_ORGS, FsV2Constants.R1_TOT_CREDIT_RATE_GE_20_ACCT_CNT, FsV2Constants.R1_MAX_WJQ_JG_USERATE, 
        FsV2Constants.R1_AVG_WJQ_ZH_USERATE, FsV2Constants.R1_AVG_WJQ_ZH_TERMS, FsV2Constants.R1_AVG_WJQ_ZH_BALANCE, FsV2Constants.QRY_TOT_RSN_DKSP_ORGS_1M, FsV2Constants.QRY_TOT_RSN_DBZGSC_CNT_24M, FsV2Constants.QRY_TOT_ORG_YHJG_XYKSP_ORGS_12M, FsV2Constants.QRY_TOT_ORG_TP_YHJG_ORGS_12M, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_RATE_6M_12M, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_RATE_3M_6M, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_RATE_3M_12M, 
        FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_6M, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_3M, FsV2Constants.QRY_TOT_ORG_TP_XFJRGS_ORGS_12M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_RATE_6M_12M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_RATE_3M_6M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_RATE_3M_12M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_6M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_3M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_ORGS_12M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_CNT_6M, 
        FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_CNT_3M, FsV2Constants.QRY_TOT_ORG_TP_XEDKGS_CNT_12M, FsV2Constants.QRY_TOT_CNT_RATE_3M_6M, FsV2Constants.QRY_TOT_CNT_RATE_1M_3M, FsV2Constants.QRY_TOT_CNT_RATE_1M_12M, FsV2Constants.QRY_TOT_CNT_6M, FsV2Constants.QRY_TOT_CNT_3M, FsV2Constants.QRY_TOT_CNT_1M, FsV2Constants.QRY_TOT_CNT_12M, FsV2Constants.QRY_ORG_TP_XFJRGS_ORGS_RATE_3M_12M, 
        FsV2Constants.QRY_MIN_ORG_TP_XFJRGS_MTH_SINCE, FsV2Constants.QRY_MIN_ORG_TP_XEDKGS_MTH_SINCE, FsV2Constants.QRY_MIN_MTH_SINCE, FsV2Constants.QRY_MAX_RSN_YHJG_DKSP_MTH_SINCE, FsV2Constants.QRY_MAX_ORG_TP_XFJRGS_MTH_SINCE, FsV2Constants.QRY_MAX_MTH_SINCE, FsV2Constants.PH_LATEST_M_CNT, FsV2Constants.LOW_TERMS_TYPE_ORG_RATE_12M, FsV2Constants.LOW_BIZ_TYPE_ORG_RATE, FsV2Constants.HIGN_QUERY_ORG_CNT, 
        FsV2Constants.HIGN_QUERY_ORG_ACCT_CNT_RATE_6M_12, FsV2Constants.HIGN_QUERY_ORG_ACCT_CNT_RATE_12M_24, FsV2Constants.HIGN_QUERY_ORG_ACCT_CNT_12, FsV2Constants.HIGH_TERMS_TYPE_ACCT_CNT, FsV2Constants.HIGH_QUERY_ORG_ACCT_CNT_RATE_12M, FsV2Constants.HIGH_ORG_QRY_MIN_MTH_SINCE, FsV2Constants.HIGH_ORG_QRY_MAX_MTH_SINCE, FsV2Constants.DK_TOT_WJQFZFDKZ_GE_1_BAL, FsV2Constants.DK_TOT_FZFDK_LST_GE_1_ACCT_CNT_3M, FsV2Constants.DK_TOT_ACCT_ST_WJQFFD_YHCKJG_BAL, 
        FsV2Constants.DK_TOT_ACCT_ST_WJQFFD_XYDB_BAL, FsV2Constants.DK_MAX_FZFDK_GE_1ZH_OVD_AMT_6M, FsV2Constants.DK_MAX_FFD_OVD_LEVEL_6M, FsV2Constants.DK_FZFDK_LST_OVD_JJ_M_CNT, FsV2Constants.DK_AVG_FFD_OVD_AMT_3M, FsV2Constants.D1_WJQ_FZFDK_GE_10K_ACCT_CNT, FsV2Constants.D1_MAX_WJQ_FZFDK_ACCT_ZJYCSHK_MTH_SINCE, FsV2Constants.ADDR_LATEST_M_CNT });
  
  private static final List<String> DEFAULT_INTEGER_VAR = Arrays.asList(new String[] { FsV2Constants.DK_TOT_FZFDK_LST_GE_1_ACCT_CNT_3M, FsV2Constants.R1_TOT_CREDIT_RATE_GE_50_ACCT_CNT, FsV2Constants.R1_TOT_WJQ_WJFLGZJYS_ACCT_CNT, FsV2Constants.R4_TOT_YYHK_MTH_CNT_6M });
  
  private static List<String> ESCAPE_TAG_LIST = Arrays.asList(new String[] { 
        "PB030Q01", "PB040Q01", "PB020Q02", "PB040Q02", "PD03ZQ01", "PB01AQ02", "PF06AQ02", "PF06AQ01", "PB01AQ03", "PB01AQ01", 
        "PF05AQ04", "PF02AQ03", "PF02AQ02" });
  
  private static String escapeEntityRefCharacters(String raw, boolean unescaped) {
    Set<Pair<String, String>> replaceList = new HashSet<>();
    for (String tag : ESCAPE_TAG_LIST) {
      String pattern = "<" + tag + ">(.*?)</" + tag + ">";
      Pattern res = Pattern.compile(pattern);
      Matcher m = res.matcher(raw);
      while (m.find()) {
        String origin = m.group(1);
        if (unescaped)
          origin = StringEscapeUtils.unescapeXml(origin); 
        String escaped = StringEscapeUtils.escapeXml11(origin);
        if (origin.equals(escaped))
          continue; 
        String before = "<" + tag + ">" + origin + "</" + tag + ">";
        String after = "<" + tag + "><![CDATA[" + escaped + "]]></" + tag + ">";
        replaceList.add(Pair.of(before, after));
      } 
    } 
    for (Pair<String, String> item : replaceList)
      raw = raw.replace((CharSequence)item.getLeft(), (CharSequence)item.getRight()); 
    return raw;
  }
  
  private static PersonCreditReport parseRawXml(String content) throws PbocException {
    try {
      return PersonCreditReport.valueOf(content);
    } catch (Exception e) {
      try {
        String exceptionScene1 = escapeEntityRefCharacters(content, false);
        return PersonCreditReport.valueOf(exceptionScene1);
      } catch (Exception ex) {
        try {
          String exceptionScene2 = escapeEntityRefCharacters(content, true);
          return PersonCreditReport.valueOf(exceptionScene2);
        } catch (Exception es) {
          throw new PbocException("解析异常", ex);
        } 
      } 
    } 
  }
  
  public static Map<String, Object> processXmlCredit(String xmlStr) throws PbocException {
    PersonCreditReport report = parseRawXml(xmlStr);
    return FsV1CalculateUtil.calc(report);
  }
  
  public static Map<String, Object> processXmlCreditV2(String xmlStr) throws PbocException {
    PersonCreditReport report = parseRawXml(xmlStr);
    Map<String, Object> result = FsV2CalculateUtil.calc(report);
    result.putAll(FsV2CalculateUtil.sqlQuery(report));
    FsV2CalculateUtil.extend(result);
    result.put("report_no", report.getReportHeaderBlock().getReportHeaderUnit().getReportSignSegment().getReportNo());
    return putDefaultValue(result);
  }
  
  public static Map<String, Object> processHtmlCreditV2(String htmlStr) throws PbocException {
    PersonCreditReport report = HandlerChainV2.parserHtml(htmlStr);
    Map<String, Object> result = FsV2CalculateUtil.calc(report);
    result.putAll(FsV2CalculateUtil.sqlQuery(report));
    FsV2CalculateUtil.extend(result);
    result.put("report_no", report.getReportHeaderBlock().getReportHeaderUnit().getReportSignSegment().getReportNo());
    return putDefaultValue(result);
  }
  
  public static Map<String, Object> putDefaultValue(Map<String, Object> result) {
    DEFAULT_LONG_VAR.forEach(x -> {
          if (result.get(x) == null)
            result.put(x, Long.valueOf(0L)); 
        });
    DEFAULT_DOUBLE_VAR.forEach(x -> {
          if (result.get(x) == null)
            result.put(x, Double.valueOf(0.0D)); 
        });
    DEFAULT_INTEGER_VAR.forEach(x -> {
          if (result.get(x) == null) {
            result.put(x, Long.valueOf(0L));
          } else {
            result.put(x, Long.valueOf(((Integer)result.get(x)).intValue()));
          } 
        });
    return result;
  }
}
